@@ -9,19 +9,28 @@ public class MapPosition
 	private static final int MAP_TILE_SIZE = 256;
 
 	/** x position (scale depends on zoom) */
-	private int _xPosition = 0;
+	private double _xPosition = 0.0;
 	/** y position (scale depends on zoom) */
-	private int _yPosition = 0;
+	private double _yPosition = 0.0;
 
 	/** Zoom level, from 2 to max */
 	private int _zoom = 12;
 	/** Factor to zoom by, 2 to the power of zoom */
 	private int _zoomFactor = 1 << _zoom;
+	/** Scaling factor on screen (usually 1.0) */
+	private double _displayScaling = 1.0;
 
 	/** Maximum zoom level */
 	private static final int MAX_ZOOM = 21;
 
 
+	/**
+	 * @param inScaling current display scaling factor from OS
+	 */
+	public void setDisplayScaling(double inScaling) {
+		_displayScaling = inScaling;
+	}
+
 	/**
 	 * Zoom and pan to show the selected area
 	 * @param inMinX minimum transformed X
@@ -33,7 +42,6 @@ public class MapPosition
 	 */
 	public void zoomToXY(double inMinX, double inMaxX, double inMinY, double inMaxY, int inWidth, int inHeight)
 	{
-		// System.out.println("Zooming to " + inMinX + ", " + inMaxX + ", " + inMinY + ", " + inMaxY + "; width=" + inWidth + ", height=" + inHeight);
 		double diffX = Math.abs(inMaxX - inMinX);
 		double diffY = Math.abs(inMaxY - inMinY);
 		// Find out what zoom level to go to
@@ -102,8 +110,7 @@ public class MapPosition
 	 * @param inValue value to transform
 	 * @return pixels
 	 */
-	private int transformToPixels(double inValue)
-	{
+	private int transformToPixels(double inValue) {
 		return transformToPixels(inValue, _zoom);
 	}
 
@@ -113,8 +120,7 @@ public class MapPosition
 	 * @param inZoom zoom value to use
 	 * @return pixels
 	 */
-	private static int transformToPixels(double inValue, int inZoom)
-	{
+	private static int transformToPixels(double inValue, int inZoom) {
 		return (int) (inValue * MAP_TILE_SIZE * (1 << inZoom));
 	}
 
@@ -124,9 +130,8 @@ public class MapPosition
 	 * @param inWidth current width of window
 	 * @return x coordinate
 	 */
-	public double getXFromPixels(int inPixelX, int inWidth)
-	{
-		return ((inPixelX - inWidth/2) + _xPosition) * 1.0 / MAP_TILE_SIZE / _zoomFactor;
+	public double getXFromPixels(int inPixelX, int inWidth) {
+		return ((inPixelX - inWidth/2) * _displayScaling + _xPosition) / MAP_TILE_SIZE / _zoomFactor;
 	}
 
 	/**
@@ -135,9 +140,8 @@ public class MapPosition
 	 * @param inHeight current height of window
 	 * @return y coordinate
 	 */
-	public double getYFromPixels(int inPixelY, int inHeight)
-	{
-		return ((inPixelY - inHeight/2) + _yPosition) * 1.0 / MAP_TILE_SIZE / _zoomFactor;
+	public double getYFromPixels(int inPixelY, int inHeight) {
+		return ((inPixelY - inHeight/2) * _displayScaling + _yPosition) / MAP_TILE_SIZE / _zoomFactor;
 	}
 
 	/**
@@ -145,9 +149,8 @@ public class MapPosition
 	 * @param inValue value to transform
 	 * @return number of pixels right (+ve) or left (-ve) from the centre
 	 */
-	public int getXFromCentre(double inValue)
-	{
-		return transformToPixels(inValue) - _xPosition;
+	public int getXFromCentre(double inValue) {
+		return transformToPixels(inValue) - (int) _xPosition;
 	}
 
 	/**
@@ -155,9 +158,8 @@ public class MapPosition
 	 * @param inValue value to transform
 	 * @return number of pixels up (+ve) or down (-ve) from the centre
 	 */
-	public int getYFromCentre(double inValue)
-	{
-		return transformToPixels(inValue) - _yPosition;
+	public int getYFromCentre(double inValue) {
+		return transformToPixels(inValue) - (int) _yPosition;
 	}
 
 	/**
@@ -178,10 +180,10 @@ public class MapPosition
 	public int[] getTileIndices(int inWidth, int inHeight)
 	{
 		int[] result = new int[4];
-		result[0] = getTileIndex(_xPosition - inWidth/2);
-		result[1] = getTileIndex(_xPosition + inWidth/2);
-		result[2] = getTileIndex(_yPosition - inHeight/2);
-		result[3] = getTileIndex(_yPosition + inHeight/2);
+		result[0] = getTileIndex((int) _xPosition - inWidth/2);
+		result[1] = getTileIndex((int) _xPosition + inWidth/2);
+		result[2] = getTileIndex((int) _yPosition - inHeight/2);
+		result[3] = getTileIndex((int) _yPosition + inHeight/2);
 		return result;
 	}
 
@@ -194,33 +196,30 @@ public class MapPosition
 	public int[] getDisplayOffsets(int inWidth, int inHeight)
 	{
 		int[] result = new int[2];
-		result[0] = getDisplayOffset(_xPosition - inWidth/2);
-		result[1] = getDisplayOffset(_yPosition - inHeight/2);
+		result[0] = getDisplayOffset((int) _xPosition - inWidth/2);
+		result[1] = getDisplayOffset((int) _yPosition - inHeight/2);
 		return result;
 	}
 
 	/**
 	 * @return x index of the centre tile
 	 */
-	public int getCentreTileX()
-	{
-		return getTileIndex(_xPosition);
+	public int getCentreTileX() {
+		return getTileIndex((int) _xPosition);
 	}
 
 	/**
 	 * @return y index of the centre tile
 	 */
-	public int getCentreTileY()
-	{
-		return getTileIndex(_yPosition);
+	public int getCentreTileY() {
+		return getTileIndex((int) _yPosition);
 	}
 
 	/**
 	 * @param inPosition position of point
 	 * @return tile index for that point
 	 */
-	private int getTileIndex(int inPosition)
-	{
+	private int getTileIndex(int inPosition) {
 		return inPosition / MAP_TILE_SIZE;
 	}
 
@@ -228,10 +227,8 @@ public class MapPosition
 	 * @param inPosition position of point
 	 * @return pixel offset for that point
 	 */
-	private int getDisplayOffset(int inPosition)
-	{
+	private int getDisplayOffset(int inPosition) {
 		return inPosition % MAP_TILE_SIZE;
-		// I thought that &255 would be slightly faster, but it gives the wrong result
 	}
 
 	/**
@@ -242,8 +239,8 @@ public class MapPosition
 		if (_zoom < MAX_ZOOM)
 		{
 			setZoom(_zoom + 1);
-			_xPosition *= 2;
-			_yPosition *= 2;
+			_xPosition *= 2.0;
+			_yPosition *= 2.0;
 		}
 	}
 
@@ -255,8 +252,8 @@ public class MapPosition
 		if (_zoom >= 3)
 		{
 			setZoom(_zoom - 1);
-			_xPosition /= 2;
-			_yPosition /= 2;
+			_xPosition /= 2.0;
+			_yPosition /= 2.0;
 		}
 	}
 
@@ -275,8 +272,7 @@ public class MapPosition
 	 */
 	public void pan(int inDeltaX, int inDeltaY)
 	{
-		// TODO: Check bounds?
-		_xPosition += inDeltaX;
-		_yPosition += inDeltaY;
+		_xPosition += (inDeltaX * _displayScaling);
+		_yPosition += (inDeltaY * _displayScaling);
 	}
 }
